@@ -1,19 +1,49 @@
+import logging
+import re
+
 from django.conf import settings
+from django.core.exceptions import ValidationError
 from django.db.models import signals
 from django.db.models.fields import CharField, FloatField, CharField, Field
+from django import forms
 
 from django_measurement import measure, utils
 
 
+logger = logging.getLogger(__name__)
+
+
+class MeasurementFormField(forms.CharField):
+    MEASUREMENT_RE = re.compile(r'(?P<value>[0-9.]+) ?(?P<unit>[A-Za-z_]+)')
+
+    def to_python(self, value):
+        if not value:
+            return None
+        match = self.MEASUREMENT_RE.match(value)
+        if not match:
+            raise ValidationError('%s could not be parsed into a known measurement' % value)
+        value, unit = match.groups()
+        unit = unit.lower()
+        measurement = utils.guess_measurement(value, unit)
+        if not match:
+            raise ValidationError('%s is not a known unit' % unit)
+        return measurement
+
 def get_measurement_parts(value):
     if isinstance(value, measure.UnknownMeasure):
         return value.get_measurement_parts()
+    measure_name = '%s(%s)' % (
+        value.__class__.__name__,
+        value.STANDARD_UNIT
+    )
     measure_name = value.__class__.__name__
     original_unit = value._default_unit
     standard_value = getattr(value, value.STANDARD_UNIT)
     return measure_name, original_unit, standard_value
 
 class MeasurementFieldDescriptor(object):
+    MEASUREMENT_NAME_RE = re.compile(r'([a-zA-Z0-9]+)(?:\(([a-zA-Z0-9]+)\))?')
+
     def __init__(self, field, measurement_field_name, original_unit_field_name, measure_field_name):
         self.field = field
         self.measurement_field_name = measurement_field_name
@@ -24,13 +54,18 @@ class MeasurementFieldDescriptor(object):
         measures = utils.build_measure_list()
         return measures[measure_name]
 
+    def _get_measure_name_and_std_unit(self, measure_string):
+        return self.MEASUREMENT_NAME_RE.search(measure_string).groups()
+
     def __get__(self, instance, instance_type=None):
         if instance is None:
             return self
 
-        measure_name = getattr(
-            instance, 
-            self.measure_field_name
+        measure_name, std_unit = self._get_measure_name_and_std_unit(
+            getattr(
+                instance, 
+                self.measure_field_name
+            )
         )
         measurement_value = getattr(
             instance,
@@ -42,6 +77,15 @@ class MeasurementFieldDescriptor(object):
         )
         try:
             instance_measure = self._get_measure_by_name(measure_name)
+            if std_unit and instance_measure.STANDARD_UNIT != std_unit:
+                raise ValueError(
+                    'Measurement %s base unit %s does not match stored %s' %
+                    (
+                        measure_name,
+                        instance_measure.STANDARD_UNIT,
+                        std_unit
+                    )
+                )
             measurement = utils.get_measurement(
                 instance_measure,
                 measurement_value,
@@ -139,7 +183,6 @@ class MeasurementField(Field):
             kwargs[self.get_original_unit_field_name()] = original_unit
             kwargs[self.get_measurement_field_name()] = standard_value
 
-
 try:
     from south.modelsinspector import add_introspection_rules
     add_introspection_rules([], ["^django_measurement\.fields\.MeasurementField"])
